import { of, from } from 'rxjs';
import { InjectionToken, Injectable, Inject, NgModule, Directive, Input, TemplateRef, ViewContainerRef, Optional, Host, defineInjectable, inject } from '@angular/core';
import { Router } from '@angular/router';
import { map, mergeMap, reduce, tap } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const ACCESS_CONFIG = new InjectionToken('ACCESS_CONFIG');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AccessStrategy {
}
class FakeAccessStrategy extends AccessStrategy {
    /**
     * @param {?} access
     * @return {?}
     */
    has(access) {
        return of(false);
    }
}
FakeAccessStrategy.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} config
 * @param {?=} __1
 * @return {?}
 */
function flatten(config, { parse = (/**
 * @param {?} v
 * @return {?}
 */
v => v), group = false, delimiter = '.' } = {}) {
    /** @type {?} */
    const flatConfig = {};
    /**
     * @param {?} path
     * @param {?} value
     * @return {?}
     */
    function setConfig(path, value) {
        if (!flatConfig[path]) {
            flatConfig[path] = new Set();
        }
        flatConfig[path].add(value);
    }
    /**
     * @param {?} path
     * @param {?} prop
     * @return {?}
     */
    function getPath(path, prop) {
        return path
            ? path + delimiter + prop
            : prop;
    }
    /**
     * @param {?} accesses
     * @param {?} prop
     * @param {?} value
     * @param {?} path
     * @return {?}
     */
    function visitor(accesses, prop, value, path) {
        if (Array.isArray(value)) {
            value.forEach((/**
             * @param {?} access
             * @return {?}
             */
            access => {
                visitor(accesses, prop, access, path);
            }));
        }
        else if (typeof value === 'object') {
            /** @type {?} */
            const childrenAccesses = children(value, getPath(path, prop));
            if (group) {
                accesses = accesses.concat(childrenAccesses);
                accesses.forEach((/**
                 * @param {?} access
                 * @return {?}
                 */
                access => {
                    setConfig(getPath(path, access.prop), access.action);
                }));
            }
        }
        else {
            /** @type {?} */
            const expression = parse(value);
            setConfig(getPath(path, prop), expression);
            accesses = accesses.concat({ action: expression, prop });
        }
        return accesses;
    }
    /**
     * @param {?} obj
     * @param {?=} path
     * @return {?}
     */
    function children(obj, path = '') {
        return Object.keys(obj)
            .reduce((/**
         * @param {?} accesses
         * @param {?} prop
         * @return {?}
         */
        (accesses, prop) => visitor(accesses, prop, obj[prop], path)), []);
    }
    children(config);
    return flatConfig;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const Operator = {
    AND: 'AND',
    OR: 'OR',
};
/** @type {?} */
let configurationAccess = {};
/** @type {?} */
let hasAccessStrategy = (/**
 * @return {?}
 */
() => of(false));
/**
 * @param {?} config
 * @return {?}
 */
function setConfigurationAccess(config) {
    /**
     * @param {?} expression
     * @return {?}
     */
    function parse(expression) {
        return {
            list: expression.split(' && '),
            operator: Operator.AND
        };
    }
    configurationAccess = flatten(config, { parse, group: true });
}
/**
 * @param {?} accessTest
 * @return {?}
 */
function setHasAccessStrategy(accessTest) {
    hasAccessStrategy = accessTest;
}
/**
 * @param {?} expression
 * @return {?}
 */
function parse(expression) {
    /** @type {?} */
    const arr = expression.split(':');
    return {
        path: (arr[0] || '').replace(' ', ''),
        action: (arr[1] || '').replace(' ', '')
    };
}
/**
 * @param {?} accessExpression
 * @return {?}
 */
function canExpression(accessExpression) {
    /** @type {?} */
    const access = Array.isArray(accessExpression)
        ? accessExpression
        : [accessExpression];
    return from(access)
        .pipe(map((/**
     * @param {?} a
     * @return {?}
     */
    a => a.replace(':', '.'))), mergeMap((/**
     * @param {?} a
     * @return {?}
     */
    a => can(a))), reduce((/**
     * @param {?} acc
     * @param {?} value
     * @return {?}
     */
    (acc, value) => acc || value), false));
}
/**
 * @param {?} path
 * @return {?}
 */
function can(path) {
    try {
        /** @type {?} */
        const access = configurationAccess[path];
        if (!!access) {
            return testAccess(Array.from(access));
        }
        console.error(`Undefined path ${path}`);
        return of(false);
    }
    catch (e) {
        console.error(e);
        return of(false);
    }
}
/**
 * @param {?} access
 * @param {?} op
 * @param {?} predicate
 * @param {?} init
 * @return {?}
 */
function testAccessReducer(access, op, predicate, init) {
    return from((/** @type {?} */ (access)))
        .pipe(mergeMap((/**
     * @param {?} currentAccess
     * @return {?}
     */
    currentAccess => testAccess(currentAccess, op))), reduce(predicate, init));
}
/**
 * @param {?} access
 * @param {?=} op
 * @return {?}
 */
function testAccess(access, op = Operator.OR) {
    return Array.isArray(access)
        ? (op === Operator.AND)
            ? testAccessReducer(access, op, (/**
             * @param {?} acc
             * @param {?} value
             * @return {?}
             */
            (acc, value) => acc && value), true)
            : testAccessReducer(access, op, (/**
             * @param {?} acc
             * @param {?} value
             * @return {?}
             */
            (acc, value) => acc || value), false)
        : typeof access === 'string'
            ? hasAccessStrategy(access)
            : testAccess(access.list, access.operator);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccessService {
    /**
     * @param {?} config
     * @param {?} accessStrategy
     */
    constructor(config, accessStrategy) {
        setHasAccessStrategy((/**
         * @param {?} accessName
         * @return {?}
         */
        accessName => accessStrategy.has(accessName)));
        setConfigurationAccess(config.accesses || {});
    }
    /**
     * @param {?} accessExpression
     * @return {?}
     */
    can(accessExpression) {
        return canExpression(accessExpression);
    }
}
AccessService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
AccessService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [ACCESS_CONFIG,] }] },
    { type: AccessStrategy }
];
/** @nocollapse */ AccessService.ngInjectableDef = defineInjectable({ factory: function AccessService_Factory() { return new AccessService(inject(ACCESS_CONFIG), inject(AccessStrategy)); }, token: AccessService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccessPathDirective {
}
AccessPathDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxAccessPath]'
            },] }
];
AccessPathDirective.propDecorators = {
    ngxAccessPath: [{ type: Input }]
};
class AccessDirective {
    /**
     * @param {?} template
     * @param {?} viewContainer
     * @param {?} accessService
     * @param {?} parentAccessPath
     */
    constructor(template, viewContainer, accessService, parentAccessPath) {
        this.template = template;
        this.viewContainer = viewContainer;
        this.accessService = accessService;
        this.parentAccessPath = parentAccessPath;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /** @type {?} */
        let ngxAccess = this.ngxAccess;
        if (this.parentAccessPath) {
            const { path, action } = parse(this.parentAccessPath.ngxAccessPath);
            if (this.ngxAccess) {
                if (Array.isArray(this.ngxAccess)) {
                    ngxAccess = `${this.ngxAccess.map((/**
                     * @param {?} access
                     * @return {?}
                     */
                    access => access.replace('$', path)))}:${action}`;
                }
                else {
                    const { path: childPath, action: childAction } = parse(ngxAccess);
                    ngxAccess = `${childPath.replace('$', path)}:${childAction ? childAction : action}`;
                }
            }
            else {
                ngxAccess = `${path}:${action}`;
            }
        }
        this.accessService.can(ngxAccess)
            .subscribe((/**
         * @param {?} access
         * @return {?}
         */
        access => access
            ? this.viewContainer.createEmbeddedView(this.template)
            : this.ngxAccessElse
                ? this.viewContainer.createEmbeddedView(this.ngxAccessElse)
                : null));
    }
}
AccessDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxAccess]'
            },] }
];
/** @nocollapse */
AccessDirective.ctorParameters = () => [
    { type: TemplateRef },
    { type: ViewContainerRef },
    { type: AccessService },
    { type: AccessPathDirective, decorators: [{ type: Optional }, { type: Host }] }
];
AccessDirective.propDecorators = {
    ngxAccess: [{ type: Input }],
    ngxAccessElse: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccessGuard {
    /**
     * @param {?} router
     * @param {?} accessService
     * @param {?} config
     */
    constructor(router, accessService, config) {
        this.router = router;
        this.accessService = accessService;
        this.config = config;
    }
    /**
     * @param {?} next
     * @return {?}
     */
    canActivate(next) {
        return this.accessService.can(next.data.expression)
            .pipe(tap((/**
         * @param {?} hasAccess
         * @return {?}
         */
        hasAccess => !hasAccess && this.router.navigate([next.data.redirect || this.config.redirect]))));
    }
}
AccessGuard.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
AccessGuard.ctorParameters = () => [
    { type: Router },
    { type: AccessService },
    { type: undefined, decorators: [{ type: Inject, args: [ACCESS_CONFIG,] }] }
];
/** @nocollapse */ AccessGuard.ngInjectableDef = defineInjectable({ factory: function AccessGuard_Factory() { return new AccessGuard(inject(Router), inject(AccessService), inject(ACCESS_CONFIG)); }, token: AccessGuard, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccessModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: AccessModule,
            providers: [
                {
                    provide: ACCESS_CONFIG,
                    useValue: {
                        accesses: config.accesses || {},
                        redirect: config.redirect || '/unauthorized',
                    }
                },
                config.strategy || {
                    provide: AccessStrategy,
                    useClass: FakeAccessStrategy
                }
            ]
        };
    }
}
AccessModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AccessPathDirective, AccessDirective],
                exports: [AccessPathDirective, AccessDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AccessModule, AccessPathDirective, AccessDirective, setConfigurationAccess, setHasAccessStrategy, parse, canExpression, flatten, AccessGuard, AccessService, AccessStrategy, FakeAccessStrategy, ACCESS_CONFIG, AccessDirective as ɵb, AccessPathDirective as ɵa, AccessStrategy as ɵd, FakeAccessStrategy as ɵe, AccessService as ɵc };

//# sourceMappingURL=ngx-access.js.map